use rusqlite::Connection;
use std::path::{Path, PathBuf};
use std::rc::Rc;

use crate::ui::cards::new::*;
use crate::widgets::card_entry::CardEntry;
use glib::subclass::InitializingObject;
use gtk::glib::object::ObjectExt;
use gtk::glib::{clone, closure_local};
use gtk::subclass::prelude::*;
use gtk::{glib, Button, CompositeTemplate, ScrolledWindow, SearchEntry, Window};
use gtk::{prelude::*, FileDialog};
use sha256::try_digest;

#[derive(CompositeTemplate, Default)]
#[template(resource = "/org/foxarmy/learn-hieroglyph/cards/edit/ui.xml")]
pub struct MemoryCardsEditScene {
    #[template_child]
    pub search_entry: TemplateChild<SearchEntry>,
    #[template_child]
    pub add_button: TemplateChild<Button>,
    #[template_child]
    pub first: TemplateChild<CardEntry>,
    #[template_child]
    pub cards: TemplateChild<ScrolledWindow>,
}

#[glib::object_subclass]
impl ObjectSubclass for MemoryCardsEditScene {
    const NAME: &'static str = "MemoryCardsEditScene";
    type Type = super::MemoryCardsEditScene;
    type ParentType = gtk::ApplicationWindow;

    fn class_init(klass: &mut Self::Class) {
        klass.bind_template();
    }

    fn instance_init(obj: &InitializingObject<Self>) {
        obj.init_template();
    }
}

impl ObjectImpl for MemoryCardsEditScene {
    fn constructed(&self) {
        self.parent_constructed();

        let binding = self.obj();

        self.add_button.connect_closure("clicked", 
                                        false,
                                        closure_local!(@strong binding => move |_b: &Button| {
                                        let new_win = Rc::new(MemoryCardsNewScene::new(&binding.application().unwrap()));
                                        new_win.get_file_choose_button().connect_clicked(clone!(@strong new_win => move |_| {
                                            gtk::glib::MainContext::default().spawn_local(file_choose_dialog(Rc::clone(&new_win)));
                                        }));
                                        new_win.present();
        }));

        let c: &ScrolledWindow = self.cards.as_ref();
        c.set_child(glib::bitflags::__private::core::option::Option::Some(
            &CardEntry::new(&"a".to_string(), &"a".to_string()),
        ));
    }
}

async fn file_choose_dialog<W: IsA<gtk::Window>>(window: Rc<W>) {
    let dialog: FileDialog = gtk::FileDialog::builder().build();
    let answer = dialog.open_future(Some(&*window)).await;
    let path = answer.unwrap().path().unwrap();
    let path: &str = path.as_path().to_str().unwrap();
    let w: &MemoryCardsNewScene = Into::<&Window>::into(window.upcast_ref())
        .downcast_ref()
        .unwrap(); // Weird casting from &Window as passed in func to &MemoryCardsNewScene
    w.get_image_widget().set_file(Some(path));
    let binding = Path::new(path);
    let hash = try_digest(binding).unwrap();

    w.get_done_button().connect_closure("clicked", false, closure_local!(@strong w => move |_b: &Button| {
        let hieroglyph = w.get_hieroglyph_input();
        let reading = w.get_reading_input();
        let program_home_path = &*crate::db::get_program_home_path().read().unwrap().clone();
        let conn = Connection::open(PathBuf::new().join(program_home_path).join("db.sqlite")).unwrap();

        let query = "INSERT INTO cards (image_hash, hieroglyph, translation) VALUES(?1, ?2, ?3)";
        conn.execute(query, (&hash, &hieroglyph, &reading)).unwrap();
        w.close();
    }));
}

impl WidgetImpl for MemoryCardsEditScene {}

impl WindowImpl for MemoryCardsEditScene {}

impl ApplicationWindowImpl for MemoryCardsEditScene {}
